@@ -9,11 +9,14 @@ use apexl\Io\modules\payment\entities\paymentLogEntity;
 use Calcinai\OAuth2\Client\Provider\Xero;
 use XeroPHP\Application as XeroApp;
 use XeroPHP\Models\Accounting\Account;
+use XeroPHP\Models\Accounting\Address;
 use XeroPHP\Models\Accounting\BrandingTheme;
 use XeroPHP\Models\Accounting\Contact;
+use XeroPHP\Models\Accounting\CreditNote;
 use XeroPHP\Models\Accounting\History;
 use XeroPHP\Models\Accounting\Invoice;
 use XeroPHP\Models\Accounting\Payment;
+use XeroPHP\Models\Accounting\Phone;
 use XeroPHP\Webhook;
 use Psr\Http\Message\ServerRequestInterface as Request;
 
@@ -74,7 +77,13 @@ class xeroProvider implements providerInterface {
                     if ($invoice = $this->getInvoice($item->provider_id)) {
                                     if ($invoiceEntity->paid != '1') {
                                     $invoiceEntity->paid = $invoice->AmountDue < 0.01 ? 1 : 0;
+                                        if ($invoiceEntity->paid == 1) {
+                                            $this->invoiceService->reenableInvoiceSubscriptions($invoiceEntity);
+                                        }
                                     }
+
+                        $invoiceEntity = $this->syncCreditNoteInfo($invoice, $invoiceEntity);
+
                                     $invoiceEntity->store();
                         $result = true;
                                 }
@@ -85,6 +94,7 @@ class xeroProvider implements providerInterface {
                                         if ($invoiceEntity = $this->invoiceService->getInvoiceByReference($invoice->Reference)) {
                                             if (trim($invoiceEntity->provider_id) == "") {
                                     $invoiceEntity->provider_id = $item->provider_id;
+                                    $invoiceEntity = $this->syncCreditNoteInfo($invoice, $invoiceEntity);
                                                 $invoiceEntity->store();
                                             }
                                 $result = true;
@@ -98,6 +108,24 @@ class xeroProvider implements providerInterface {
         return $result;
     }
 
+    protected function syncCreditNoteInfo($invoice, $invoiceEntity)
+    {
+        try {
+            if ($invoice->getAmountCredited() > 0) {
+                $creditInfo = [];
+                foreach ($invoice->getCreditNotes() as $creditNote) {
+                    $total = number_format($creditNote->getTotal(), 2);
+                    $creditInfo[] = $creditNote->getCreditNoteNumber() . " ({$total})";
+                }
+                $invoiceEntity->credit_note_info = implode("<br>", $creditInfo);
+            }
+        } catch (\Exception $e) {
+            error_log('Sync Credit Note Failed: '.$e->getMessage());
+        }
+
+        return $invoiceEntity;
+    }
+
     protected function logRegest($log)
     {
         $paymentLog = new paymentLogEntity();
@@ -110,7 +138,7 @@ class xeroProvider implements providerInterface {
         $paymentLog->store();
     }
 
-    public function getInvoices($email)
+    public function getInvoices($email, $expanded=false)
     {
         $this->instantiate();
 
@@ -142,7 +170,7 @@ class xeroProvider implements providerInterface {
 
             if (isset($invoices) && is_iterable($invoices)) {
         foreach ($invoices as $invoice) {
-            $data[] = [
+                    $dat = [
                 'id' => $invoice->getInvoiceID(),
                 'name' => $invoice->getInvoiceNumber(),
                 'email' => $email,
@@ -153,6 +181,27 @@ class xeroProvider implements providerInterface {
                 'total' => number_format($invoice->getTotal(), 2),
                 //$this->getInvoiceUrl($invoice) ?? 'Unavailable',
             ];
+
+                    if ($expanded) {
+                        $dat['dateMysql'] = $invoice->getDate()->format('Y-m-d');
+                        $dat['paid'] = $invoice->getAmountDue() < 0.01 ? 1 : 0;
+                        $dat['lineItems'] = [];
+                        foreach ($invoice->getLineItems() as $lineItem) {
+                            $item = (object)[
+                                'description' => $lineItem->getDescription(),
+                                'accountCode' => $lineItem->getAccountCode(),
+                                'quantity' => $lineItem->getQuantity(),
+                                'perUnit' => $lineItem->getUnitAmount(),
+                                'amount' => $lineItem->getLineAmount(),
+                                'vat' => $lineItem->getTaxAmount(),
+                            ];
+                            $dat['lineItems'][] = $item;
+                        }
+
+                        $dat['lineItems'] = json_encode($dat['lineItems']);
+                    }
+
+                    $data[] = $dat;
         }
             }
         }
@@ -227,6 +276,49 @@ class xeroProvider implements providerInterface {
         die();
     }
 
+    public function changeInvoiceContact($invoiceId, $toContactName)
+    {
+        $invoice = $this->getInvoice($invoiceId);
+        if ($invoice) {
+            $contacts = $this->getContact((object)['name' => $toContactName], 'name');
+            if ($contacts) {
+                foreach($contacts as $contact) {
+                    echo '<pre>';
+                    print_r($contact);
+                    echo '</pre>';
+                $invoice->setContact($contact);
+                $response = $invoice->save();
+                    echo '<pre>';
+                    print_r($response);
+                    echo '</pre>';
+                    die();
+                }
+            }
+        }
+            }
+
+    public function sendInvoiceEmail($invoiceId)
+    {
+        $this->instantiate();
+
+        $invoice = $this->getInvoice($invoiceId);
+        if ($invoice) {
+            try {
+                $log = [
+                    'api_name' => 'Invoice',
+                    'method_name' => 'sendEmail',
+                    'request_data' => [
+                        'InvoiceId' => $invoiceId,
+                    ]
+                ];
+                $result = $invoice->sendEmail();
+            } catch (\Exception $e) {
+                $log['response_data'] = [$e->getMessage()];
+            }
+            $this->logRegest($log);
+        }
+    }
+
     public function createInvoice($lineitems, $contactData, $lookupContactBy='name', $lineAmountType='Inclusive', $dueDate = null, $desc = '', $branding = '', $sendEmail=FALSE, $reference=null, $type = 'ACCREC', $currency = 'GBP') {
 
         $this->instantiate();
@@ -274,7 +366,7 @@ class xeroProvider implements providerInterface {
                 'method_name' => 'save',
                 'request_data' => [
                     'Status' => "AUTHORISED",
-                    'LineAmountType' => 'Inclusive',
+                    'LineAmountType' => $lineAmountType,
                     'Type' => $type,
                     'CurrencyCode' => $currency,
                     'DueDate' => $dueDateTime->format('Y-m-d'),
@@ -332,6 +424,106 @@ class xeroProvider implements providerInterface {
         return $paymentId;
     }
 
+    public function creditInvoice($invoiceId, $contactData, $lookupContactBy)
+    {
+        if ($invoice = $this->getInvoice($invoiceId)) {
+            if ($creditNoteId = $this->createCreditNote($invoice->getLineItems(), $contactData, $lookupContactBy)) {
+                $creditNotes = $this->xero->load(CreditNote::class)
+                    ->where('CreditNoteID', $creditNoteId)
+                    ->execute();
+                foreach ($creditNotes as $creditNote) {
+                    try {
+                        $log = [
+                            'api_name' => 'CreditNoteAllocation',
+                            'method_name' => 'save',
+                            'request_data' => [
+                                'Amount' => $invoice->getTotal(),
+                                'InvoiceId' => $invoiceId,
+                            ]
+                        ];
+
+                        $allocation = (new CreditNote\Allocation($this->xero))
+                            ->setAppliedAmount($invoice->getTotal())
+                            ->setInvoice($invoice)
+                            ->setDate(new \DateTime('now', new \DateTimeZone('Europe/London')));
+
+                        $creditNote->addAllocation($allocation);
+                        $response = $creditNote->save();
+
+                    } catch (\Exception $e) {
+                        $log['response_data'] = [$e->getMessage()];
+                    }
+                    $this->logRegest($log);
+                }
+            }
+        }
+
+        return $creditNoteId ?? null;
+    }
+
+    public function createCreditNote($lineItems, $contactData, $lookupContactBy, $lineAmountType = 'Exclusive')
+    {
+        /*
+         {
+  "Type": "ACCPAYCREDIT",
+  "Status": "AUTHORISED",
+  "Contact": {
+    "ContactID": "eaa28f49-6028-4b6e-bb12-d8f6278073fc"
+  },
+  "Date": "2009-03-29",
+  "LineAmountTypes": "Exclusive",
+  "LineItems": [
+    {
+      "Description": "MacBook - White",
+      "Quantity": 1.0000,
+      "UnitAmount": 1995.00,
+      "AccountCode": "720"
+    }
+  ]
+}
+         */
+
+        $this->instantiate();
+
+        $xeroContacts = $this->getContact($contactData, $lookupContactBy);
+        $xeroContact = $xeroContacts[0] ?? null;
+
+        try {
+            $log = [
+                'api_name' => 'CreditNote',
+                'method_name' => 'save',
+                'request_data' => [
+                    'Date' => date('Y-m-d'),
+                    'Type' => 'ACCRECCREDIT',
+                    'ContactId' => $xeroContact->getContactID(),
+                    'LineAmountType' => $lineAmountType,
+                    'LineItems' => $lineItems,
+                    'Status' => 'AUTHORISED'
+                ]
+            ];
+
+            $creditNote = (new CreditNote($this->xero))
+                ->setType('ACCRECCREDIT')
+                ->setStatus('AUTHORISED')
+                ->setDate(new \DateTime('now'))
+                ->setContact($xeroContact)
+                ->setLineAmountType($lineAmountType);
+
+            foreach ($lineItems as $lineItem) {
+                $creditNote->addLineItem($lineItem);
+            }
+
+            $response = $creditNote->save();
+            $creditNoteId = $creditNote->getCreditNoteID();
+        } catch (\Exception $e) {
+            $log['response_data'] = [$e->getMessage()];
+            $creditNoteId = null;
+        }
+        $this->logRegest($log);
+
+        return $creditNoteId;
+    }
+
     public function getInvoice($invoiceId)
     {
         $this->instantiate();
@@ -406,7 +598,7 @@ class xeroProvider implements providerInterface {
         return $data;
     }
 
-    protected function createContact($details){
+    public function createContact($details){
         $this->instantiate();
 
         try {
@@ -419,11 +611,61 @@ class xeroProvider implements providerInterface {
                     'LastName' => $details->lastName,
                     'EmailAddress' => $details->emailAddress,
                     'TaxNumber' => $details->taxNumber,
+                    'Phone' => $details->phone ?? null,
                 ]
             ];
 
             $contact = new \XeroPHP\Models\Accounting\Contact($this->xero);
             $contact->setName($details->name)->setFirstName($details->firstName)->setLastName($details->lastName)->setEmailAddress($details->emailAddress)->setTaxNumber($details->taxNumber);
+
+            if (isset($details->phone)) {
+                $phone = new Phone($this->xero);
+                $phone->setPhoneType('DEFAULT');
+                $phone->setPhoneNumber($details->phone);
+                $contact->addPhone($phone);
+            }
+
+            $contact->save();
+
+            if (isset($details->address)) {
+                $this->addAddressToContact($contact, $details->address);
+            }
+        } catch (\Exception $e) {
+            $log['response_data'] = [$e->getMessage()];
+        }
+        $this->logRegest($log);
+
+
+        return $contact;
+    }
+
+    function addAddressToContact(Contact $contact, array $addressData)
+    {
+        $this->instantiate();
+
+        try {
+            $log = [
+                'api_name' => 'Address',
+                'method_name' => 'save',
+                'request_data' => $addressData
+            ];
+
+            $addressType = $addressData['AddressType'] ?? 'POBOX';
+
+            // Create new address
+            $newAddress = new Address($this->xero);
+            $newAddress
+                ->setAddressType($addressType)
+                ->setAddressLine1($addressData['AddressLine1'] ?? '')
+                ->setAddressLine2($addressData['AddressLine2'] ?? '')
+                ->setAddressLine3($addressData['AddressLine3'] ?? '')
+                ->setCity($addressData['City'] ?? '')
+                ->setRegion($addressData['Region'] ?? '')
+                ->setPostalCode($addressData['PostalCode'] ?? '')
+                ->setCountry($addressData['Country'] ?? '');
+
+            $contact->addAddress($newAddress);
+            // Save the updated contact
             $contact->save();
         } catch (\Exception $e) {
             $log['response_data'] = [$e->getMessage()];
