@@ -6,6 +6,7 @@ use apexl\Config\Singleton;
 use apexl\Io\includes\Hook;
 use apexl\Io\includes\Routes;
 use apexl\Io\includes\System;
+use apexl\Io\modules\email\services\emailService;
 use apexl\Io\modules\email\services\templateService;
 use apexl\Io\modules\invoice\entities\invoiceEntity;
 use apexl\Io\modules\invoice\entities\invoicePaymentEntity;
@@ -20,11 +21,11 @@ use apexl\Io\modules\product\entities\productEntity;
 use apexl\Io\modules\subscription\entities\subscriptionCreditEntity;
 use apexl\Io\modules\subscription\entities\subscriptionEntity;
 use apexl\Io\modules\subscription\services\subscriptionService;
+use apexl\Io\modules\user\entities\userContactDetailsEntity;
 use apexl\Io\modules\user\entities\userEntity;
 use apexl\Io\modules\user\services\currentUser;
 use apexl\Io\services\Database;
 use apexl\Io\services\Logger;
-use apexl\Io\services\Mailer;
 use apexl\Vault\Vault;
 use Psr\Http\Message\ResponseInterface as Response;
 use Psr\Http\Message\ServerRequestInterface as Request;
@@ -67,7 +68,7 @@ class invoiceService {
      * @return invoiceEntity
      * @throws \Exception
      */
-    public function createInvoice($userId, $lineItems, $sendReceipt=false, $subscriptions=[], $paymentProvider=null, $remoteOrderId=null, $rollForwardSubscriptions=true, $billingEnabled=true, $brandingThemeId=null, $lookupContactBy='email', $sendEmail=FALSE,$lineAmountType='Exclusive', $contactData=null)
+    public function createInvoice($userId, $lineItems, $sendReceipt=false, $subscriptions=[], $paymentProvider=null, $remoteOrderId=null, $rollForwardSubscriptions=true, $billingEnabled=true, $brandingThemeId=null, $lookupContactBy='email', $sendEmail=FALSE,$lineAmountType='Exclusive', $contactData=null, $sendToProvider=true)
     {
         if (count($lineItems) > 0) {
 
@@ -86,6 +87,12 @@ class invoiceService {
             if (!$billingEnabled) $invoice->billing_enabled = 0;
             $invoice->send_receipt = $sendReceipt;
             $invoice->payment_provider = $paymentProvider;
+            $invoice->send_to_provider = $sendToProvider ? 1 : 0;
+            if (!$brandingThemeId && isset($this->config->app->invoicing->xero->branding_themes->default)) {
+                $brandingThemeId = $this->config->app->invoicing->xero->branding_themes->default;
+            }
+            $brandingTheme = $brandingThemeId ?? $this->config->app->invoicing->xero->default_branding_theme;
+            $invoice->provider_branding_id = $brandingTheme;
             $invoice->store();
             $invReference = str_replace('{ID}', $invoice->id, REMOTE_INVOICE_REF);
 
@@ -109,40 +116,25 @@ class invoiceService {
 
             if ($this->provider) {
                 if (!$contactData) {
-                $userEntity = new userEntity();
-                $userEntity->load($userId);
-                $contactData = (object)[
-                    'name' => trim($userEntity->first_name . ' ' . $userEntity->last_name),
-                    'firstName' => $userEntity->first_name,
-                    'lastName' => $userEntity->last_name,
-                    'emailAddress' => $userEntity->email,
-                    'taxNumber' => ""
-                ];
-                    if (isset($userEntity->invoice_name_override) && trim($userEntity->invoice_name_override) != "") {
-                        $contactData->name = $userEntity->invoice_name_override;
-                        $lookupContactBy = 'name';
-                    }
+                    list($contactData, $lookupContactBy) = $this->getProviderContactDataByUser($userId, $lookupContactBy);
                 }
 
-                if (!$brandingThemeId && isset($this->config->app->invoicing->xero->branding_themes->default)) {
-                    $brandingThemeId = $this->config->app->invoicing->xero->branding_themes->default;
-                }
-                $brandingTheme = $brandingThemeId ?? $this->config->app->invoicing->xero->default_branding_theme;
+                if ($sendToProvider) {
+                    try {
+                        list($invoiceId, $invoiceNumber, $contactData) = $this->createProviderInvoice($lineItems, $contactData, $lookupContactBy, null, '', $brandingTheme, $sendEmail, $invReference, 'ACCREC', 'GBP', $lineAmountType);
 
-                try {
-                    list($invoiceId, $invoiceNumber, $contactData) = $this->createProviderInvoice($lineItems, $contactData, $lookupContactBy, null, '', $brandingTheme, $sendEmail, $invReference, 'ACCREC', 'GBP', $lineAmountType);
+                    $this->providerContactData[$invoiceNumber] = $contactData;
 
-                $this->providerContactData[$invoiceNumber] = $contactData;
-
-                $invoice->provider_id = $invoiceId;
-                $invoice->invoice_number = $invoiceNumber;
-                $invoice->store();
-                } catch (\Exception $e) {
-                    $invoice->provider_error = $e->getMessage();
-                    $invoice->on_hold = 1;
+                    $invoice->provider_id = $invoiceId;
+                    $invoice->invoice_number = $invoiceNumber;
                     $invoice->store();
+                    } catch (\Exception $e) {
+                        $invoice->provider_error = $e->getMessage();
+                        $invoice->on_hold = 1;
+                        $invoice->store();
 
-                    $this->providerErrorAlert($invoice);
+                        $this->providerErrorAlert($invoice);
+                    }
                 }
             }
         }
@@ -151,6 +143,47 @@ class invoiceService {
         return $invoice;
     }
 
+    protected function getProviderContactDataByUser($userId, $lookupContactBy)
+    {
+        $userEntity = new userEntity();
+        $userEntity->load($userId);
+        $contactData = (object)[
+            'name' => trim($userEntity->first_name . ' ' . $userEntity->last_name),
+            'firstName' => $userEntity->first_name,
+            'lastName' => $userEntity->last_name,
+            'emailAddress' => $userEntity->email,
+            'taxNumber' => ""
+        ];
+
+        $userContactDetails = new userContactDetailsEntity();
+        $userContactDetails->load($userId);
+        if (isset($userContactDetails->phone) && $userContactDetails->phone) {
+            $contactData->phone = $userContactDetails->phone;
+        }
+        if (isset($userContactDetails->shipping_address_1) && ($userContactDetails->shipping_address_1 || $userContactDetails->billing_address_1)) {
+            $addressType = $userContactDetails->same_address == 1 ? 'shipping' : 'billing';
+            if ($userContactDetails->{$addressType.'_address_1'}) {
+                $contactData->address = [
+                    'AddressType'   => 'POBOX',
+                    'AddressLine1'  => $userContactDetails->{$addressType.'_address_1'},
+                    'AddressLine2'  => $userContactDetails->{$addressType.'_address_2'},
+                    'AddressLine3'  => $userContactDetails->{$addressType.'_address_3'},
+                    'City'          => $userContactDetails->{$addressType.'_town'},
+                    'Region'        => $userContactDetails->{$addressType.'_county'},
+                    'PostalCode'    => $userContactDetails->{$addressType.'_postcode'},
+                    'Country'    => $userContactDetails->{$addressType.'_country'},
+                ];
+            }
+        }
+
+        if (isset($userEntity->invoice_name_override) && trim($userEntity->invoice_name_override) != "") {
+            $contactData->name = $userEntity->invoice_name_override;
+            $lookupContactBy = 'name';
+        }
+
+        return [$contactData, $lookupContactBy];
+    }
+
     public function getProviderContactData($invoiceNumber)
     {
         return $this->providerContactData[$invoiceNumber] ?? null;
@@ -183,7 +216,16 @@ class invoiceService {
                     'taxNumber' => ""
                 ];
 
-                list($invoiceId, $invoiceNumber, $contactData) = $this->createProviderInvoice($lineItems, $contactData, $lookupContactBy, null, '', $this->config->app->invoicing->xero->branding_themes->default ?? $this->config->app->invoicing->xero->default_branding_theme);
+
+
+                    $brandingTheme = $this->config->app->invoicing->xero->branding_themes->default ?? $this->config->app->invoicing->xero->default_branding_theme;
+                //$brandingTheme = 'd68d8639-e9ba-4f7b-a603-a53ac6e3db38';
+
+                if (isset($invoiceEntity->provider_branding_id) && trim($invoiceEntity->provider_branding_id) != "") {
+                    $brandingTheme = trim($invoiceEntity->provider_branding_id);
+                }
+
+                list($invoiceId, $invoiceNumber, $contactData) = $this->createProviderInvoice($lineItems, $contactData, $lookupContactBy, null, '', $brandingTheme);
 
                 $result->invoices_sent++;
 
@@ -253,8 +295,8 @@ class invoiceService {
             'description' => $description,
             'accountCode' => $overrideNominalCode ? $overrideNominalCode : $nominalCode,
             'quantity' => $quantity,
-            'perUnit' => !in_array($taxType, ['ECZROUTPUT', 'ZERORATEDOUTPUT']) ? $perUnitCost : number_format(1.2*$perUnitCost, 2, '.', ''),
-            'amount' => !in_array($taxType, ['ECZROUTPUT', 'ZERORATEDOUTPUT']) ? $total : number_format(1.2*$totalRaw, 2, '.', ''),
+            'perUnit' => !in_array($taxType, ['ECZROUTPUT', 'ZERORATEDOUTPUT']) ? $perUnitCost : number_format($perUnitCost, 2, '.', ''),
+            'amount' => !in_array($taxType, ['ECZROUTPUT', 'ZERORATEDOUTPUT']) ? $total : number_format($totalRaw, 2, '.', ''),
             'vat' => !in_array($taxType, ['NONE', 'ECZROUTPUT', 'ZERORATEDOUTPUT']) ? number_format(0.2*$totalRaw, 2, '.', '') : 0,
             'productId' => $productId
         ];
@@ -316,6 +358,9 @@ class invoiceService {
      */
     public function triggerSubscriptionInvoices(Request $request, Response $response, $args)
     {
+        $pauseBilling = System::getVariable('io_invoice_pause_billing') ?? null;
+        if ($pauseBilling == '1') return System::asJson($response, ['subscriptionsProcessed' => 0, 'reason' => 'billing paused by io_invoice_pause_billing variable']);
+
         if (date('H') < 8 || date('H') > 17) return System::asJson($response, ['subscriptionsProcessed' => 0, 'reason' => 'before 8am or after 6pm']);
 
         if (!self::checkStartCron()) return System::asJson($response, ['error' => 'exit due to cron duplication. process: '.getmypid()]);
@@ -450,6 +495,27 @@ class invoiceService {
         return $result;
     }
 
+    public function triggerProviderUploads(Request $request, Response $response, $args)
+    {
+        if (!self::checkStartCron('provider_uploads')) return System::asJson($response, ['error' => 'exit due to cron duplication. process: '.getmypid()]);
+
+        try {
+            $invoiceService = new invoiceService();
+            $invoiceEntity = new invoiceEntity();
+            $invoices = $invoiceEntity->getRequiringProviderUpload(1);
+            foreach ($invoices as $invoice) {
+                if ($invoice->send_to_provider == '1' && trim($invoice->provider_id) == '') {
+                    if (!self::checkContinueCron('provider_uploads')) return System::asJson($response, ['error' => 'exit due to cron duplication. process: ' . getmypid()]);
+                    $invoiceService->sendInvoiceToProvider($invoice->id);
+                }
+            }
+        } catch (\Exception $e) {
+            $this->genericAlert('URGENT - GlobalM2MSIM provider invoice uploads error', "There has been an error with the invoice provider upload triggering process:<br><br> {$e->getMessage()}");
+        }
+
+        return true;
+    }
+
     public function triggerPaymentAlerts(Request $request, Response $response, $args)
     {
 
@@ -463,6 +529,11 @@ class invoiceService {
                 'varName' => 'cron_invoice_payments_completed',
                 'triggerDays' => '2',
                 'alert' => 'Cron Invoice Charging failed to complete for more than two daya'
+            ],
+            (object)[
+                'varName' => 'cron_provider_uploads_completed',
+                'triggerDays' => '2',
+                'alert' => 'Cron automated provider invoice uploads failed to complete for more than two daya'
             ]
         ];
 
@@ -488,16 +559,17 @@ class invoiceService {
                 'subject' => 'URGENT: GlobalM2MSIM Dashboard Cron failures',
                 'message' => '<p>'.implode("<br />", $alerts).'</p>',
             ];
-            $from = $this->config->app->site->email_address ?? 'no-reply@localhost.com';
-            $fromName = $this->config->app->site->name ?? 'localhost';
-            $mailer = new Mailer();
-            $mailer->setFrom($from, $fromName);
-            $mailer->addAddress("mike@developmentx.uk");     // Add a recipient
 
-            $mailer->Subject = 'URGENT: GlobalM2MSIM Dashboard Cron failures';
-            $mailer->Body = templateService::fetch("generic", $emailData);
-            $mailer->IsHTML(true);
-            $result = $mailer->send();
+            $emailService = new emailService();
+            $result = $emailService->send(
+                (object) [
+                    'to' => "[WEBMASTER]",
+                    'subject' => 'URGENT: GlobalM2MSIM Dashboard Cron failures',
+                    'body' => templateService::fetch("generic", $emailData),
+                    'user_id' => 0,
+                ],
+                'generic'
+            );
 
             System::setVariable('payment_alert_email_sent', date('U'));
 
@@ -514,16 +586,17 @@ class invoiceService {
             'subject' => 'High Priority: GlobalM2MSIM Dashboard Invoice not sent to Provider',
             'message' => "<p>Invoice Local ID: {$invoiceEntity->id} could not be sent to Xero.<br><br>Customer name: {$userEntity->getNiceName()}.</p><p>We need to investigate the reason for this, and ensure the invoice is billed correctly.</p>",
         ];
-        $from = $this->config->app->site->email_address ?? 'no-reply@localhost.com';
-        $fromName = $this->config->app->site->name ?? 'localhost';
-        $mailer = new Mailer();
-        $mailer->setFrom($from, $fromName);
-        $mailer->addAddress("mike@developmentx.uk");     // Add a recipient
 
-        $mailer->Subject = 'High Priority: GlobalM2MSIM Dashboard Invoice not sent to Provider';
-        $mailer->Body = templateService::fetch("generic", $emailData);
-        $mailer->IsHTML(true);
-        $result = $mailer->send();
+        $emailService = new emailService();
+        $result = $emailService->send(
+            (object) [
+                'to' => "[WEBMASTER]",
+                'subject' => 'High Priority: GlobalM2MSIM Dashboard Invoice not sent to Provider',
+                'body' => templateService::fetch("generic", $emailData),
+                'user_id' => 0,
+            ],
+            'generic'
+        );
     }
 
     protected function genericAlert($subject, $message)
@@ -532,16 +605,17 @@ class invoiceService {
             'subject' => $subject,
             'message' => "<p>$message</p>",
         ];
-        $from = $this->config->app->site->email_address ?? 'no-reply@localhost.com';
-        $fromName = $this->config->app->site->name ?? 'localhost';
-        $mailer = new Mailer();
-        $mailer->setFrom($from, $fromName);
-        $mailer->addAddress("mike@developmentx.uk");     // Add a recipient
 
-        $mailer->Subject = $subject;
-        $mailer->Body = templateService::fetch("generic", $emailData);
-        $mailer->IsHTML(true);
-        $result = $mailer->send();
+        $emailService = new emailService();
+        $result = $emailService->send(
+            (object) [
+                'to' => "[WEBMASTER]",
+                'subject' => $subject,
+                'body' => templateService::fetch("generic", $emailData),
+                'user_id' => 0,
+            ],
+            'generic'
+        );
     }
 
     protected static function checkStartCron($process='subscription')
@@ -698,9 +772,9 @@ class invoiceService {
                 $paymentEntity = $cardValidationService->capturePaymentWithToken($invoiceEntity->user_id, $invoiceEntity->payment_provider, ($invoiceEntity->total + $invoiceEntity->vat), $ref, $isSubscriptionPayment, $data, $meta, $transactionData, $userPresent);
                 if (isset($paymentEntity->id)) {
 
+                    $invoicePaymentEntity = $this->addPaymentToInvoice($invoiceEntity->id, $paymentEntity);
                     // Submit payment to Xero, or other provider
                     if ($sendPaymentInfo && trim($invoiceEntity->provider_id) != "") {
-                    $invoicePaymentEntity = $this->addPaymentToInvoice($invoiceEntity->id, $paymentEntity);
                     if (trim($invoiceEntity->provider_id) != "") {
                         if ($paymentEntity->status == 'successful') {
                             $this->submitPaymentToProvider($paymentEntity, $invoiceEntity, $invoicePaymentEntity);
@@ -734,7 +808,7 @@ class invoiceService {
         }
     }
 
-    public function reenableInvoiceSubscriptions($invoiceEntity, $reEnableSubscriptions = true)
+    public function reenableInvoiceSubscriptions($invoiceEntity, $reEnableSubscriptions=true, $foreceEnableAll=false)
     {
         Logger::log('reenable_invoice_subscriptions_start', ['invoice_id' => $invoiceEntity->id, 'reEnableSubscriptions' => $reEnableSubscriptions]);
 
@@ -744,10 +818,10 @@ class invoiceService {
 
         Logger::log('reenable_invoice_subscriptions_data', ['invoice_id' => $invoiceEntity->id, 'reEnableSubscriptions' => $reEnableSubscriptions, 'subscriptions' => $subscriptions]);
 
-        if ($reEnableSubscriptions && is_array($subscriptions)) {
+        if (($reEnableSubscriptions || $foreceEnableAll) && is_array($subscriptions)) {
             foreach ($subscriptions as $subscription) {
                 Logger::log('reenable_invoice_subscriptions_in_loop', ['invoice_id' => $invoiceEntity->id, 'subscription' => $subscription]);
-                if ($subscription->enabled == '1' && trim($subscription->cancellation_date) != "") {
+                if ($subscription->enabled == '1' && trim($subscription->cancellation_date) != "" || $foreceEnableAll) {
                     Logger::log('reenable_invoice_subscriptions_in_if', ['invoice_id' => $invoiceEntity->id, 'subscription' => $subscription]);
                     $subscriptionEntity = new subscriptionEntity();
                     $subscriptionEntity->load($subscription->id);
@@ -769,7 +843,7 @@ class invoiceService {
         $subscriptionService = new subscriptionService();
         $subscriptions = $subscriptionEntity->getByInvoiceId($invoiceEntity->id);
         $cancellationDate = new \DateTime('now');
-        $cancellationDate->add(new \DateInterval('P3M')); // Bill SIMs for a further 3 months
+        $cancellationDate->add(new \DateInterval('P2M')); // Bill SIMs for a further 2 months
         $subscriptionIds = [];
         foreach ($subscriptions as $subscription) {
             $subscriptionIds[] = $subscription->id;
@@ -815,14 +889,16 @@ class invoiceService {
             $customData = Hook::processHook('pre_send_payment_failure_notification', $subscriptionIds);
             if (is_array($customData)) $emailData = array_merge($customData, $emailData);
 
-            $mailer = new Mailer();
-            $mailer->setFrom($from, $fromName);
-            $mailer->addAddress($user->email);     // Add a recipient
-
-            $mailer->Subject = 'Payment Failure Notification';
-            $mailer->Body = templateService::fetch("payment_failure", $emailData);
-            $mailer->IsHTML(true);
-            $result = $mailer->send();
+            $emailService = new emailService();
+            $result = $emailService->send(
+                (object) [
+                    'to' => $user->email,
+                    'subject' => 'Payment Failure Notification',
+                    'body' => templateService::fetch("payment_failure", $emailData),
+                    'user_id' => $user->id,
+                ],
+                'payment_failure'
+            );
 
             Logger::log('payment_failure_notification', ['sent' => $result, 'user' => $user->getData()]);
 
@@ -878,9 +954,11 @@ class invoiceService {
             $invoiceEntity->load($invoicePaymentEntity->invoice_id);
         }
         if (isset($invoiceEntity->provider_id) && $invoiceEntity->provider_id) { // Submit payment information to provider
-            $providerPaymentId = $this->provider->createPayment($invoiceEntity->provider_id, $paymentEntity->amount, $paymentEntity->provider_ref_2, $this->config->app->invoicing->sale_account_code);
-            $invoicePaymentEntity->provider_payment_id = $providerPaymentId;
-            $invoicePaymentEntity->store();
+            if (!isset($this->config->app->invoicing->xero->dont_send_payments) || !in_array($invoiceEntity->payment_provider, $this->config->app->invoicing->xero->dont_send_payments)) {
+                $providerPaymentId = $this->provider->createPayment($invoiceEntity->provider_id, $paymentEntity->amount, $paymentEntity->provider_ref_2, $this->config->app->invoicing->sale_account_code);
+                $invoicePaymentEntity->provider_payment_id = $providerPaymentId;
+                $invoicePaymentEntity->store();
+            }
         }
     }
 
@@ -964,22 +1042,31 @@ class invoiceService {
                     'total' => number_format($invoiceEntity->total + $invoiceEntity->vat, 2),
                 ];
 
-                $mailer = new Mailer();
-                $mailer->setFrom($from, $fromName);
-                $mailer->addAddress($userEntity->email);     // Add a recipient
-
-                $mailer->Subject = 'GeoSIM - Your Receipt';
-                $mailer->Body = templateService::fetch("receipt_email", $emailData);
-
                 // Attach invoice
+                $attachments = [];
                 if ($this->provider) {
                     if ($content = $this->provider->getInvoicePDF($invoiceEntity->provider_id)){
-                        $mailer->addStringAttachment($content, $invoiceEntity->invoice_number . '.pdf');
+                        $attachments[] = (object)[
+                            'content' => $content,
+                            'filename' => $invoiceEntity->invoice_number . '.pdf'
+                        ];
                     }
                 }
 
-                $mailer->IsHTML(true);
-                if ($result = $mailer->send()) {
+                $emailService = new emailService();
+                $result = $emailService->send(
+                    (object) [
+                        'to' => $userEntity->email,
+                        'subject' => 'GeoSIM - Your Receipt',
+                        'body' => templateService::fetch("receipt_email", $emailData),
+                        'user_id' => $userEntity->id,
+                        'attachments' => $attachments,
+                    ],
+                    'receipt_email'
+                );
+
+
+                if ($result) {
                     $invoiceEntity->receipt_sent_date = date('Y-m-d H:i:s');
                     $invoiceEntity->store();
                 }
@@ -1171,6 +1258,37 @@ class invoiceService {
         }
     }
 
+    public function creditProviderInvoice($invoiceEntity)
+    {
+        if ($this->provider && $invoiceEntity->provider_id) {
+            list($contactData, $lookupContactBy) = $this->getProviderContactDataByUser($invoiceEntity->user_id, 'email');
+            return $this->provider->creditInvoice($invoiceEntity->provider_id, $contactData, $lookupContactBy);
+        }
+
+        return null;
+    }
+
+    public function createProviderContact($userId, $invoiceName=null)
+    {
+
+        list($localContactData, $lookupContactBy) = $this->getProviderContactDataByUser($userId, 'email');
+        if ($invoiceName) $localContactData->name = $invoiceName;
+
+        $addContact = TRUE;
+        $xeroContact = $this->provider->getContact($localContactData, $lookupContactBy);
+
+        // check existing / create new contact
+        foreach($xeroContact as $contact){
+            if(!empty($contact)) {
+                $addContact = FALSE; // Contact found - don't add a new one
+                break;
+            }
+        }
+        if($addContact) $contact = $this->provider->createContact($localContactData); //  create a new contact
+
+        return $addContact;
+    }
+
 
     public function getInvoiceUrl($invoiceId)
     {
